package com.xzzn.ems.service.impl;

import com.xzzn.common.utils.StringUtils;
import com.xzzn.common.utils.bean.BeanUtils;
import com.xzzn.ems.domain.EmsBatteryCluster;
import com.xzzn.ems.domain.EmsBatteryData;
import com.xzzn.ems.domain.EmsBatteryStack;
import com.xzzn.ems.domain.EmsPcsData;
import com.xzzn.ems.domain.vo.*;
import com.xzzn.ems.mapper.*;
import com.xzzn.ems.service.ISingleSiteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

/**
 * 单点监控 服务层实现
 */
@Service
public class SingleSiteServiceImpl implements ISingleSiteService {

    @Autowired
    private EmsPcsDataMapper emsPcsDataMapper;
    @Autowired
    private EmsAlarmRecordsMapper emsAlarmRecordsMapper;
    @Autowired
    private EmsBatteryDataMapper emsBatteryDataMapper;
    @Autowired
    private EmsBatteryStackMapper emsBatteryStackMapper;
    @Autowired
    private EmsBatteryClusterMapper emsBatteryClusterMapper;
    @Autowired
    private EmsPcsBranchDataMapper emsPcsBranchDataMapper;

    @Override
    public SiteMonitorHomeVo getSiteMonitorDataVo(Long siteId) {
        SiteMonitorHomeVo siteMonitorHomeVo = new SiteMonitorHomeVo();

        if (siteId != null) {
            // 实时告警数据 名称+状态+告警内容
            List<SiteMonitorHomeAlarmVo> siteMonitorHomeAlarmVo = emsAlarmRecordsMapper.getAlarmRecordsBySiteId(siteId);
            siteMonitorHomeVo.setSiteMonitorHomeAlarmVo(siteMonitorHomeAlarmVo);
            // 能量数据
            List<SiteMonitorDataVo> siteMonitorDataVoList = emsPcsDataMapper.getPcsDataBySiteId(siteId);
            if (!CollectionUtils.isEmpty(siteMonitorDataVoList)) {
                BigDecimal dayChargeCap = new BigDecimal(0);
                BigDecimal dayDisChargeCap = new BigDecimal(0);
                BigDecimal totalChargeCap = new BigDecimal(0);
                BigDecimal totalDisChargeCap = new BigDecimal(0);
                for (SiteMonitorDataVo sitePcsData : siteMonitorDataVoList) {
                    // 总充电量
                    totalChargeCap = totalChargeCap.add(sitePcsData.getChargedCap());
                    // 总放电量
                    totalDisChargeCap = totalDisChargeCap.add(sitePcsData.getDisChargedCap());
                    // 获取当天的充电量+放电量
                    String pcsDate = sitePcsData.getAmmeterDate();
                    boolean isToday= checkIsToday(pcsDate);
                    if(isToday){
                        dayChargeCap = dayChargeCap.add(sitePcsData.getChargedCap());
                        dayDisChargeCap = dayDisChargeCap.add(sitePcsData.getDisChargedCap());
                    }
                }
                siteMonitorHomeVo.setDayChargedCap(dayChargeCap);
                siteMonitorHomeVo.setDayDisChargedCap(dayDisChargeCap);
                siteMonitorHomeVo.setTotalChargedCap(totalChargeCap);
                siteMonitorHomeVo.setTotalDischargedCap(totalDisChargeCap);
                // 储能可用电量
                BigDecimal energyStorageAvailElec = siteMonitorHomeVo.getTotalDischargedCap().subtract(siteMonitorHomeVo.getTotalChargedCap());
                siteMonitorHomeVo.setEnergyStorageAvailElec(energyStorageAvailElec);
            }
            siteMonitorHomeVo.setSiteMonitorDataVo(siteMonitorDataVoList);
            // 电网实时功率
            siteMonitorHomeVo.setGridNrtPower(emsPcsDataMapper.getGridNrtPower(siteId));
        }

        return siteMonitorHomeVo;
    }

    private boolean checkIsToday(String pcsDate) {
        boolean flag = false;
        Calendar calendar = Calendar.getInstance();
        int todayMonth = calendar.get(Calendar.MONTH) + 1;
        int todayDay = calendar.get(Calendar.DAY_OF_MONTH);
        if (StringUtils.isNotEmpty(pcsDate)){
            String[] pcsDateArray = pcsDate.split("/");
            if (todayMonth == Integer.parseInt(pcsDateArray[0]) &&
                    todayDay == Integer.parseInt(pcsDateArray[1])) {
                flag = true;
            }
        }
        return flag;
    }

    // 获取单站监控实时运行头部数据
    @Override
    public SiteMonitorRuningHeadInfoVo getSiteRunningHeadInfo(Long siteId) {
        SiteMonitorRuningHeadInfoVo siteMonitorRunningHeadInfoVo = new SiteMonitorRuningHeadInfoVo();

        if (siteId != null) {
            // 实时有功功率/实时无功功率/今日充电量/今日放电量
            siteMonitorRunningHeadInfoVo = emsPcsDataMapper.getSiteRunningHeadInfo(siteId);
            // 电池簇SOC
            // 电池堆SOH
            EmsBatteryData emsBatteryData = emsBatteryDataMapper.getBatteryDataBySiteId(siteId);
            if (emsBatteryData != null) {
                siteMonitorRunningHeadInfoVo.setSoc(emsBatteryData.getSoc());
                siteMonitorRunningHeadInfoVo.setSoh(emsBatteryData.getSoh());
            }
        }

        return siteMonitorRunningHeadInfoVo;
    }

    //  获取单站监控实时运行曲线图数据
    @Override
    public SiteMonitorRuningInfoVo getRunningGraph(Long siteId) {
        SiteMonitorRuningInfoVo siteMonitorRuningInfoVo = new SiteMonitorRuningInfoVo();
        if (siteId != null) {
            //储能功率list
            //pcs平均温度list
            //电池平均soclist
            //电池平均温度list
        }
        return siteMonitorRuningInfoVo;
    }

    // 根据site_id获取pcs详细数据+支路数据
    @Override
    public List<PcsDetailInfoVo> getPcsDetailInfo(Long siteId) {
        List<PcsDetailInfoVo> pcsDetailInfoVoList = new ArrayList<>();

        if (siteId != null) {
            // 获取pcs最新数据
            List<EmsPcsData> emsPcsDataList = emsPcsDataMapper.getPcsDetailInfoBySiteId(siteId);
            if (!CollectionUtils.isEmpty(emsPcsDataList)) {
                for (EmsPcsData sitePcsData : emsPcsDataList) {
                    // 赋值
                    PcsDetailInfoVo pcsDetailInfoVo = new PcsDetailInfoVo();
                    BeanUtils.copyProperties(sitePcsData, pcsDetailInfoVo);

                    Long deviceId = sitePcsData.getDeviceId();
                    // 获取支路最新数据
                    if (deviceId != null) {
                        List<PcsBranchInfo> pcsBranchInfoList = emsPcsBranchDataMapper.getPcsBranchInfoList(siteId, deviceId);
                        pcsDetailInfoVo.setPcsBranchInfoList(pcsBranchInfoList);
                    }
                    pcsDetailInfoVoList.add(pcsDetailInfoVo);
                }
            }
        }
        return pcsDetailInfoVoList;
    }

    // 获取BMS总览数据
    @Override
    public BMSOverViewVo getBMSOverView(Long siteId) {
        BMSOverViewVo  bmsOverViewVo = new BMSOverViewVo();
        EmsBatteryStack emsBatteryStack = emsBatteryStackMapper.selectEmsBatteryStackBySiteId(siteId);
        BeanUtils.copyProperties(emsBatteryStack, bmsOverViewVo);
        return bmsOverViewVo;
    }

    // 获取BMS电池簇数据
    @Override
    public BMSBatteryClusterVo getBMSBatteryCluster(Long siteId) {
        BMSBatteryClusterVo  bmsBatteryClusterVo = new BMSBatteryClusterVo();
        EmsBatteryCluster emsBatteryCluster = emsBatteryClusterMapper.getBMSBatteryCluster(siteId);
        BeanUtils.copyProperties(emsBatteryCluster, bmsBatteryClusterVo);
        return bmsBatteryClusterVo;
    }

}
